"""

This library is an API wrapper for Yandex Delivery

Author: Egor Udalov
Latest version: 1.0.1

Quickstart guide: https://yandex.ru/dev/logistics/api/about/quickstart.html

"""

from .delivery_api import YandexDeliveryApi
from . import base_methods
