from setuptools import setup

setup(name='yandex-delivery-api',
      version='1.0.3',
      description='Python wrapper for Yandex Delivery API',
      packages=['yandex_delivery'],
      author_email='edm.rud3nko@yandex.ru',
      zip_safe=False)
